# import sys
# setting path
# sys.path.append('../src')
# sys.path.append("/Users/pierrebouvet/Documents/Code/HDF5_BLS/src")

from HDF5_BLS import Wraper, Treat
import matplotlib.pyplot as plt
from scipy import optimize, stats
import numpy as np
import os

def lorentzian(nu,A,B,nu0,Gamma):
    return B + A*(Gamma/2)**2/((nu-nu0)**2+(Gamma/2)**2)

filepath = '/Users/pierrebouvet/Documents/Code/HDF5_BLS/test/test_data/GHOST_example.DAT'

hdf5 = Wraper()
hdf5.open_data(filepath)

scan_amplitude = float(hdf5.attributes["SPECTROMETER.Scan_Amplitude"])
hdf5.define_abscissa(-scan_amplitude/2, scan_amplitude/2, hdf5.data.shape[-1])

treat = Treat()

opt, std = treat.fit_model(hdf5.abscissa,
                hdf5.data,
                7.43,
                1,
                normalize = True, 
                model = "Lorentz", 
                fit_S_and_AS = True, 
                window_peak_find = 1, 
                window_peak_fit = 3, 
                correct_elastic = False)


print(opt, std)



import h5py
import numpy as np
import csv
import os
from PIL import Image

from src.Load_data import Load_Data

BLS_HDF5_Version = 0.1

class Wraper:
    def __init__(self):
        self.filepath = None
        self.attributes = {}
        self.abscissa = None
        self.raw_data = None
        self.calibration_curve = None
        self.impulse_response = None
        self.loader = Load_Data()

    def define_abscissa(self, min_val, max_val, nb_samples):
        """Defines a new abscissa axis based on min, max values, and number of samples.
    
        Parameters
        ----------
        min_val : float                           
            First point of the abscissa
        max_val : float                           
            Last point of the abscissa
        nb_sambles : float                           
            Number of samples in the abscissa
        
        Returns
        -------
        self.abscissa : numpy array
            The abscissa values
        """
        self.abscissa = np.linspace(min_val, max_val, nb_samples)
        return self.abscissa

    def export_properties_data(self, filepath_csv):
        """Exports properties to a CSV file.
    
        Parameters
        ----------
        filepath_csv : str                           
            The filepath to the csv storing the properties of the measure. This csv is meant to be made once to store all the properties of the spectrometer and then minimally adjusted to the sample being measured.
        
        Returns
        -------
        boolean : boolean
            True if the file was exported properly, false if an error occured.
        """
        try:
            with open(filepath_csv, mode='w', newline='') as csv_file:
                csv_writer = csv.writer(csv_file)
                for key, value in self.attributes.items():
                    csv_writer.writerow([key, value])
            return True
        except:
            return False

    def import_abscissa(self, filepath):
        """Imports abscissa points from a file and returns the associated array.
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the values of the abscissa
        
        Returns
        -------
        self.abscissa : numpy array
            The numpy array corresponding to the abscissa being imported
        """
        self.abscissa, _ = self.loader.load_general(filepath)
        return self.abscissa
  
    def import_properties_data(self, filepath):
        """Imports properties from a CSV file into a dictionary.
    
        Parameters
        ----------
        filepath_csv : str                           
            The filepath to the csv storing the properties of the measure. This csv is meant to be made once to store all the properties of the spectrometer and then minimally adjusted to the sample being measured.
        
        Returns
        -------
        self.attributes : dic
            The dictionnary containing all the attributes
        """
        with open(filepath, mode='r') as csv_file:
            csv_reader = csv.reader(csv_file)
            for row in csv_reader:
                key, value = row
                self.attributes[key] = value
        return self.attributes

    def open_calibration(self, filepath):
        """Opens a calibration curve file and returns the calibration curve.
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the calibration curve
        
        Returns
        -------
        self.calibration_curve : numpy array
            The numpy array corresponding to the calibration curve
        """
        self.calibration_curve, _ = self.loader.load_general(filepath)
        return self.calibration_curve

    def open_hdf5_files(self, filepath):
        """Opens a hdf5 file
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the hdf5 file
        
        Returns
        -------
        self.data : numpy array
            The numpy array corresponding to the data of the hdf5 file
        self.attributes: dic
            The dictionnary containing all the attributes of the hdf5 file opened
        """
        return self.loader.load_hdf5_file(filepath)

    def open_data(self, filepath):
        """Opens a raw data file based on its file extension and stores the filepath.
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the raw data file
        
        Returns
        -------
        self.data : numpy array
            The numpy array corresponding to the data of the hdf5 file
        self.attributes: dic
            The dictionnary containing all the attributes of the hdf5 file opened
        """
        self.filepath = filepath
        self.data, self.attributes = self.loader.load_general(filepath)
        
        return self.data, self.attributes

    def open_IR(self, filepath):
        """Opens an impulse response file and returns the curve.
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the impulse response curve
        
        Returns
        -------
        self.impulse_response : numpy array
            The numpy array corresponding to the impulse response curve
        """
        self.impulse_response, _ = self.loader.load_general(filepath)
        return self.impulse_response

    def properties_data(self, **kwargs):
        """Creates a dictionary with the given properties.
    
        Parameters
        ----------
        **kwargs : dic, optional                           
            The arguments to update or set the attributes of the hdf5 file.
        
        Returns
        -------
        self.attributes : dic
            The dictionnary containing all the attributes of the hdf5 file
        """
        self.attributes = kwargs
        return self.attributes

    def save_hdf5_as(self, filepath):
        """Saves the data and attributes to an HDF5 file.
    
        Parameters
        ----------
        save_filepath : str                           
            The filepath where to save the hdf5 file
        
        Returns
        -------
        boolean : boolean
            True if the file was saved correctly, False if not
        """
        try:
            with h5py.File(filepath, 'w') as hdf5_file:
                # Save attributes
                for key, value in self.attributes.items():
                    hdf5_file.attrs[key] = value
                
                # Save datasets if they exist
                if self.raw_data is not None:
                    hdf5_file.create_dataset('Raw_Data', data=self.raw_data)
                    print("added Raw data")
                if self.abscissa is not None:
                    hdf5_file.create_dataset('Abscissa', data=self.abscissa)
                if self.calibration_curve is not None:
                    hdf5_file.create_dataset('Calibration_Curve', data=self.calibration_curve)
                if self.impulse_response is not None:
                    hdf5_file.create_dataset('Impulse_Response', data=self.impulse_response)
            return True
        except:
            return False





import numpy as np
import os
from PIL import Image
import h5py

BLS_HDF5_Version = 0.1

class Load_Data():
    def __init__(self):
        pass

    def load_dat_file(self, filepath):
        """Loads files obtained with the GHOST software
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the GHOST file
        
        Returns
        -------
        data : np.array
            The data stored in the file
        attributes : dic
            A dictionnary with all the properties that could be recovered from the file
        """
        metadata = {}
        data = []
        name, _ = os.path.splitext(filepath)
        attributes = {}
        
        with open(filepath, 'r') as file:
            lines = file.readlines()
            # Extract metadata
            for line in lines:
                if line.strip() == '':
                    continue  # Skip empty lines
                if any(char.isdigit() for char in line.split()[0]):
                    break  # Stop at the first number
                else:
                    # Split metadata into key-value pairs
                    if ':' in line:
                        key, value = line.split(':', 1)
                        metadata[key.strip()] = value.strip()
            # Extract numerical data
            for line in lines:
                if line.strip().isdigit():
                    data.append(int(line.strip()))
        data = np.array(data)
        attributes['FILEPROP.BLS_HDF5_Version'] = BLS_HDF5_Version
        attributes['FILEPROP.Name'] = name
        attributes['MEASURE.Sample'] = metadata["Sample"]
        attributes['SPECTROMETER.Scanning_Strategy'] = "point_scanning"
        attributes['SPECTROMETER.Type'] = "TFP"
        attributes['SPECTROMETER.Illumination_Type'] = "CW"
        attributes['SPECTROMETER.Detector_Type'] = "Photon Counter"
        attributes['SPECTROMETER.Filtering_Module'] = "None"
        attributes['SPECTROMETER.Wavelength_nm'] = metadata["Wavelength"]
        attributes['SPECTROMETER.Scan_Amplitude'] = metadata["Scan amplitude"]
        spectral_resolution = float(float(metadata["Scan amplitude"])/data.shape[-1])
        attributes['SPECTROMETER.Spectral_Resolution'] = str(spectral_resolution)
        return data, attributes

    def load_tiff_file(self, filepath):
        """Loads files obtained with the GHOST software
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the tif image
        
        Returns
        -------
        data : np.array
            The data stored in the file
        attributes : dic
            A dictionnary with all the properties that could be recovered from the file
        """
        data = []
        name, _ = os.path.splitext(filepath)
        attributes = {}

        im = Image.open(filepath)
        data = np.array(im)

        attributes['FILEPROP.BLS_HDF5_Version'] = BLS_HDF5_Version
        attributes['FILEPROP.Name'] = name

        return data, attributes

    def load_hdf5_file(self, filepath):
        """Loads HDF5 files
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the HDF5 file
        
        Returns
        -------
        data : np.array
            The data stored in the file
        attributes : dic
            A dictionnary with all the properties that could be recovered from the file
        """
        attributes = {}
        with h5py.File(filepath, 'r') as hdf5_file:
            data = hdf5_file["Raw_Data"][:]
            
            for key in hdf5_file.attrs.keys():
                attributes[key] = hdf5_file.attrs[key]
            
        return data, attributes

    def load_general(self, filepath):
        """Loads files based on their extensions
    
        Parameters
        ----------
        filepath : str                           
            The filepath to the file
        
        Returns
        -------
        data : np.array
            The data stored in the file
        attributes : dic
            A dictionnary with all the properties that could be recovered from the file
        """
        _, file_extension = os.path.splitext(filepath)
        
        if file_extension.lower() == ".dat":
            # Load .DAT file format data
            return self.load_dat_file(filepath)
        elif file_extension.lower() == ".tif":
            # Load .TIFF file format data
            return self.load_tiff_file(filepath)
        elif file_extension.lower() == ".h5":
            # Load .TIFF file format data
            return self.load_hdf5_file(filepath)
        else:
            raise ValueError(f"Unsupported file format: {file_extension}")


import numpy as np
from scipy import optimize


class Treat():
    def __init__(self):
        self.treat_steps = []
    
    def remove_offset(self, data, nb_points = 10):
        """Automatically identifies the offset of the signal and removes it by identifying the regions of points that are closer to zero and removing their average.
    
        Parameters
        ----------
        data : numpy array                           
            The data that is going to be treated

        Returns
        -------
        data_treat : numpy array
            The data where the offset has been removed
        """
        pos_min = np.argmin(data)
        window = [max(0, pos_min-nb_points), min(data.size, pos_min+nb_points)]
        offset = np.average(data[window[0]:window[1]])
        data_treat = data - offset
        return data_treat

    def normalize_data(self, data, window = 10, peak_pos = -1, remove_offset = True):
        """Normalizes a data array to an amplitude of 1 after removing the offset
    
        Parameters
        ----------
        data : numpy array                           
            The data that we want to normalize
        window : int, optional 
            The window width around the position of the peak to refine its position and use its amplitude to normalize the signal. Default is 10
        peak_pos : int, optional 
            The position of the peak in the data array. Defaults to the maximal value of the spectrum
        remove_offset : bool, optional 
            Wether to remove the offset of the data or not before normalizing. Defaults to True.

        Returns
        -------
        data_treat : numpy array
            The data where the offset has been removed
        """
        if remove_offset: data_treat = self.remove_offset(data)
        if peak_pos == -1: peak_pos = np.argmax(data)
        window = [max(0, peak_pos-window), min(data.size, peak_pos+window)]
        val = np.max(data[window[0]:window[1]])
        return data_treat/val
    
    def fit_model(self, frequency, data, center_frequency, linewidth, normalize = True, model = "Lorentz", fit_S_and_AS = True, window_peak_find = 1, window_peak_fit = 3, correct_elastic = False):
        """_summary_

        Parameters
        ----------
        frequency : numpy array
            The freaquency axis corresponding to the data
        data : numpy array
            The data to fit
        center_frequency : float
            The estimate expected shift frequency
        linewidth : float
            The expected linewidth
        normalize : bool, optional
            Wether a normalization is to be made on the data before treatment or not , by default True
        model : str, optional
            The model with which to fit the data. Accepted models are "Lorentz" and "DHO", by default "Lorentz"
        fit_S_and_AS : bool, optional
            Wether to fit both the Stokes and Anti-Stokes peak during the fit (and return corresponding averaged values and propagated errors), by default True
        window_peak_find : float, optional
            The width in GHz where to find the peak, by default 1GHz
        window_peak_fit : float, optional
            The width in GHz of the windo used to fit the peak, by default 3 GHz
        correct_elastic : bool, optional
            Wether to correct for the presence of an elastic peak by setting adding a linear function to the model, by default False

        Returns
        -------
        optimal_parameters: tuple
            The returned optimal parameters for the fit (offset, amplitude, center_frequency, linewidth) averaged if both the Stokes and anti-Stokes peaks are used
        variance: tuple
            The returned variance on the fitted parameters (offset, amplitude, center_frequency, linewidth)
        """
        def lorentzian(nu, b, a, nu0, gamma):
            return b + a*(gamma/2)**2/((nu-nu0)**2+(gamma/2)**2)
        
        def lorentzian_elastic(nu, ae, be, a, nu0, gamma):
            return be + ae*nu + a*(gamma/2)**2/((nu-nu0)**2+(gamma/2)**2)
        
        def DHO(nu, b, a, nu0, gamma):
            return b + a*(gamma*nu0**2)/((nu**2-nu0**2)**2+gamma*nu0**2)
        
        def DHO_elastic(nu, ae, be, a, nu0, gamma):
            return be + ae*nu + a*(gamma*nu0**2)/((nu**2-nu0**2)**2+gamma*nu0**2)
        
        # Refine the position of the peak with a quadratic polynomial fit
        if fit_S_and_AS:
            center_frequency = abs(center_frequency)
            window_peak_find_S = np.where(np.abs(frequency+center_frequency)<window_peak_find/2)
            pol_temp_S = np.polyfit(frequency[window_peak_find_S], data[window_peak_find_S],2)
            center_frequency_S = -pol_temp_S[1]/(2*pol_temp_S[0])
            window_peak_find_AS = np.where(np.abs(frequency-center_frequency)<window_peak_find/2)
            pol_temp_AS = np.polyfit(frequency[window_peak_find_AS], data[window_peak_find_AS],2)
            center_frequency_AS = -pol_temp_AS[1]/(2*pol_temp_AS[0])

            center_frequency = center_frequency_S
        else:
            window_peak_find = np.where(np.abs(frequency-center_frequency)<window_peak_find/2)
            pol_temp = np.polyfit(frequency[window_peak_find], data[window_peak_find],2)
            center_frequency = -pol_temp[1]/(2*pol_temp[0])

        # Normalize the data is not already done
        if normalize: data = self.normalize_data(data, peak_pos=np.argmin(np.abs(frequency-center_frequency)))

        # Apply the fit
        if fit_S_and_AS:
            # Define the initial parameters of the fit
            if correct_elastic: 
                p0_S = [0, 0, 1, center_frequency_S, linewidth]
                p0_AS = [0, 0, 1, center_frequency_AS, linewidth]
                model = model+"_e"
            else: 
                p0_S = [0, 1, center_frequency_S, linewidth]
                p0_AS = [0, 1, center_frequency_AS, linewidth]

            # Define the fit function
            models = {"Lorentz": lorentzian, "DHO": DHO, "Lorentz_e": lorentzian_elastic, "DHO_e": DHO_elastic}
            f = models[model]

            # Define the windows of fit
            window_S = np.where(np.abs(frequency-center_frequency_S)<window_peak_fit/2)
            window_AS = np.where(np.abs(frequency-center_frequency_AS)<window_peak_fit/2)

            # Apply the fit on both peaks
            popt_S, pcov_S = optimize.curve_fit(f,
                                                frequency[window_S],
                                                data[window_S],
                                                p0_S)
        
            popt_AS, pcov_AS = optimize.curve_fit(f,
                                                  frequency[window_AS],
                                                  data[window_AS],
                                                  p0_AS)
            
            # Extract the errors in the form of standard deviations
            std_S = np.sqrt(np.diag(pcov_S))
            std_AS = np.sqrt(np.diag(pcov_AS))
            
            std = np.sqrt(std_AS**2+std_S**2)
            popt = 0.5*(np.array(popt_S)+np.array(popt_AS))
            popt[-2] = 0.5*(popt_AS[-2] - popt_S[-2])
        
        else:
            if correct_elastic: 
                p0 = [0, 0, 1, center_frequency, linewidth]
                model = model+"_e"
            else: 
                p0 = [0, 1, center_frequency, linewidth]
            
            # Define the fit function
            models = {"Lorentz": lorentzian, "DHO": DHO, "Lorentz_e": lorentzian_elastic, "DHO_e": DHO_elastic}
            f = models[model]

            window = np.where(np.abs(frequency-center_frequency)<window_peak_fit/2)

            popt, pcov = optimize.curve_fit(f,
                                            frequency[window],
                                            data[window],
                                            p0)
            
            std = np.sqrt(np.diag(pcov))

        return popt, std



from setuptools import setup, find_packages
from os import path
working_directory = path.abspath(path.dirname(__file__))

with open(path.join(working_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='HDF5_BLS', # name of pack which will be package dir below project
    version='0.0.8',
    url='https://github.com/PierreBouvet/HDF5_BLS',
    author='Pierre Bouvet',
    author_email='pierre.bouvet@meduniwien.ac.at',
    description='A package to convert Brillouin spectra to a HDF5 file and use them',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(), 
    install_requires=[
        "h5py >= 3",
        "pillow >= 11",
        "numpy >= 2",
        "scipy >= 1.14"
        ],
) 